@@ -258,6 +258,15 @@ GType  bd_crypto_integrity_extra_get_type () {
     return type;
 }
 
+/**
+ * BDCryptoLUKSHWEncryptionType:
+ * @BD_CRYPTO_LUKS_HW_ENCRYPTION_UNKNOWN: used for unknown/unsupported hardware encryption or when
+ *                                        error was detected when getting the information
+ * @BD_CRYPTO_LUKS_HW_ENCRYPTION_SW_ONLY: hardware encryption is not configured on this device
+ * @BD_CRYPTO_LUKS_HW_ENCRYPTION_OPAL_HW_ONLY: only OPAL hardware encryption is configured on this device
+ * @BD_CRYPTO_LUKS_HW_ENCRYPTION_OPAL_HW_AND_SW: both OPAL hardware encryption and software encryption
+ *                                               (using LUKS/dm-crypt) is configured on this device
+ */
 /**
  * BDCryptoLUKSInfo:
  * @version: LUKS version
@@ -270,6 +279,7 @@ GType  bd_crypto_integrity_extra_get_type () {
  * @metadata_size: LUKS metadata size
  * @label: label of the LUKS device (valid only for LUKS 2)
  * @subsystem: subsystem of the LUKS device (valid only for LUKS 2)
+ * @hw_encryption: hardware encryption type
  */
 /**
  * bd_crypto_luks_info_free: (skip)
@@ -311,6 +321,7 @@ BDCryptoLUKSInfo* bd_crypto_luks_info_copy (BDCryptoLUKSInfo *info) {
     new_info->metadata_size = info->metadata_size;
     new_info->label = g_strdup (info->label);
     new_info->subsystem = g_strdup (info->subsystem);
+    new_info->hw_encryption = info->hw_encryption;
 
     return new_info;
 }
@@ -566,7 +577,7 @@ GType  bd_crypto_keyslot_context_get_type () {
 }
 
 static gboolean  bd_crypto_is_tech_avail_stub (BDCryptoTech tech G_GNUC_UNUSED, guint64 mode G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_is_tech_avail' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_is_tech_avail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_is_tech_avail' called, but not implemented!");
     return FALSE;
@@ -589,7 +600,7 @@ gboolean  bd_crypto_is_tech_avail (BDCryptoTech tech, guint64 mode, GError **err
 
 
 static gchar* bd_crypto_generate_backup_passphrase_stub (GError **error) {
-    g_critical ("The function 'bd_crypto_generate_backup_passphrase' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_generate_backup_passphrase' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_generate_backup_passphrase' called, but not implemented!");
     return NULL;
@@ -613,7 +624,7 @@ gchar* bd_crypto_generate_backup_passphrase (GError **error) {
 
 
 static gboolean  bd_crypto_device_is_luks_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_device_is_luks' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_device_is_luks' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_device_is_luks' called, but not implemented!");
     return FALSE;
@@ -638,7 +649,7 @@ gboolean  bd_crypto_device_is_luks (const gchar *device, GError **error) {
 
 
 static const gchar* bd_crypto_luks_status_stub (const gchar *luks_device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_status' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_status' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_status' called, but not implemented!");
     return NULL;
@@ -663,7 +674,7 @@ const gchar* bd_crypto_luks_status (const gchar *luks_device, GError **error) {
 
 
 static BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_passphrase_stub (const guint8 *pass_data G_GNUC_UNUSED, gsize data_len G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_keyslot_context_new_passphrase' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_keyslot_context_new_passphrase' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_keyslot_context_new_passphrase' called, but not implemented!");
     return NULL;
@@ -688,7 +699,7 @@ BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_passphrase (const guint8 *
 
 
 static BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_keyfile_stub (const gchar *keyfile G_GNUC_UNUSED, guint64 keyfile_offset G_GNUC_UNUSED, gsize key_size G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_keyslot_context_new_keyfile' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_keyslot_context_new_keyfile' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_keyslot_context_new_keyfile' called, but not implemented!");
     return NULL;
@@ -714,7 +725,7 @@ BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_keyfile (const gchar *keyf
 
 
 static BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_keyring_stub (const gchar *key_desc G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_keyslot_context_new_keyring' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_keyslot_context_new_keyring' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_keyslot_context_new_keyring' called, but not implemented!");
     return NULL;
@@ -741,7 +752,7 @@ BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_keyring (const gchar *key_
 
 
 static BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_volume_key_stub (const guint8 *volume_key G_GNUC_UNUSED, gsize volume_key_size G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_keyslot_context_new_volume_key' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_keyslot_context_new_volume_key' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_keyslot_context_new_volume_key' called, but not implemented!");
     return NULL;
@@ -766,7 +777,7 @@ BDCryptoKeyslotContext* bd_crypto_keyslot_context_new_volume_key (const guint8 *
 
 
 static gboolean  bd_crypto_luks_format_stub (const gchar *device G_GNUC_UNUSED, const gchar *cipher G_GNUC_UNUSED, guint64 key_size G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, guint64 min_entropy G_GNUC_UNUSED, BDCryptoLUKSVersion luks_version G_GNUC_UNUSED, BDCryptoLUKSExtra *extra G_GNUC_UNUSED,GError **error) {
-    g_critical ("The function 'bd_crypto_luks_format' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_format' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_format' called, but not implemented!");
     return FALSE;
@@ -803,7 +814,7 @@ gboolean  bd_crypto_luks_format (const gchar *device, const gchar *cipher, guint
 
 
 static gboolean  bd_crypto_luks_open_stub (const gchar *device G_GNUC_UNUSED, const gchar *name G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, gboolean read_only G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_open' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_open' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_open' called, but not implemented!");
     return FALSE;
@@ -840,7 +851,7 @@ gboolean  bd_crypto_luks_open (const gchar *device, const gchar *name, BDCryptoK
 
 
 static gboolean  bd_crypto_luks_close_stub (const gchar *luks_device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_close' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_close' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_close' called, but not implemented!");
     return FALSE;
@@ -863,7 +874,7 @@ gboolean  bd_crypto_luks_close (const gchar *luks_device, GError **error) {
 
 
 static gboolean  bd_crypto_luks_add_key_stub (const gchar *device G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, BDCryptoKeyslotContext *ncontext G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_add_key' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_add_key' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_add_key' called, but not implemented!");
     return FALSE;
@@ -891,7 +902,7 @@ gboolean  bd_crypto_luks_add_key (const gchar *device, BDCryptoKeyslotContext *c
 
 
 static gboolean  bd_crypto_luks_remove_key_stub (const gchar *device G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_remove_key' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_remove_key' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_remove_key' called, but not implemented!");
     return FALSE;
@@ -917,7 +928,7 @@ gboolean  bd_crypto_luks_remove_key (const gchar *device, BDCryptoKeyslotContext
 
 
 static gboolean  bd_crypto_luks_change_key_stub (const gchar *device G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, BDCryptoKeyslotContext *ncontext G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_change_key' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_change_key' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_change_key' called, but not implemented!");
     return FALSE;
@@ -944,7 +955,7 @@ gboolean  bd_crypto_luks_change_key (const gchar *device, BDCryptoKeyslotContext
 
 
 static gboolean  bd_crypto_luks_resize_stub (const gchar *luks_device G_GNUC_UNUSED, guint64 size G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_resize' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_resize' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_resize' called, but not implemented!");
     return FALSE;
@@ -975,7 +986,7 @@ gboolean  bd_crypto_luks_resize (const gchar *luks_device, guint64 size, BDCrypt
 
 
 static gboolean  bd_crypto_luks_suspend_stub (const gchar *luks_device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_suspend' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_suspend' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_suspend' called, but not implemented!");
     return FALSE;
@@ -998,7 +1009,7 @@ gboolean  bd_crypto_luks_suspend (const gchar *luks_device, GError **error) {
 
 
 static gboolean  bd_crypto_luks_resume_stub (const gchar *luks_device G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_resume' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_resume' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_resume' called, but not implemented!");
     return FALSE;
@@ -1024,7 +1035,7 @@ gboolean  bd_crypto_luks_resume (const gchar *luks_device, BDCryptoKeyslotContex
 
 
 static gboolean  bd_crypto_luks_kill_slot_stub (const gchar *device G_GNUC_UNUSED, gint slot G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_kill_slot' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_kill_slot' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_kill_slot' called, but not implemented!");
     return FALSE;
@@ -1051,7 +1062,7 @@ gboolean  bd_crypto_luks_kill_slot (const gchar *device, gint slot, GError **err
 
 
 static gboolean  bd_crypto_luks_header_backup_stub (const gchar *device G_GNUC_UNUSED, const gchar *backup_file G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_header_backup' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_header_backup' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_header_backup' called, but not implemented!");
     return FALSE;
@@ -1076,7 +1087,7 @@ gboolean  bd_crypto_luks_header_backup (const gchar *device, const gchar *backup
 
 
 static gboolean  bd_crypto_luks_header_restore_stub (const gchar *device G_GNUC_UNUSED, const gchar *backup_file G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_header_restore' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_header_restore' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_header_restore' called, but not implemented!");
     return FALSE;
@@ -1102,7 +1113,7 @@ gboolean  bd_crypto_luks_header_restore (const gchar *device, const gchar *backu
 
 
 static gboolean  bd_crypto_luks_set_label_stub (const gchar *device G_GNUC_UNUSED, const gchar *label G_GNUC_UNUSED, const gchar *subsystem G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_set_label' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_set_label' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_set_label' called, but not implemented!");
     return FALSE;
@@ -1127,7 +1138,7 @@ gboolean  bd_crypto_luks_set_label (const gchar *device, const gchar *label, con
 
 
 static gboolean  bd_crypto_luks_set_uuid_stub (const gchar *device G_GNUC_UNUSED, const gchar *uuid G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_set_uuid' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_set_uuid' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_set_uuid' called, but not implemented!");
     return FALSE;
@@ -1150,8 +1161,35 @@ gboolean  bd_crypto_luks_set_uuid (const gchar *device, const gchar *uuid, GErro
 }
 
 
+static gboolean  bd_crypto_luks_convert_stub (const gchar *device G_GNUC_UNUSED, BDCryptoLUKSVersion target_version G_GNUC_UNUSED, GError **error) {
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_convert' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_crypto_luks_convert' called, but not implemented!");
+    return FALSE;
+}
+
+static gboolean  (*_bd_crypto_luks_convert) (const gchar *device, BDCryptoLUKSVersion target_version, GError **error) = bd_crypto_luks_convert_stub;
+
+/**
+ * bd_crypto_luks_convert:
+ * @device:  a LUKS device to convert to a different version of LUKS
+ * @target_version: the LUKS version to convert to
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: whether the @device was converted to @target_version.
+ *          False, if the @device is already in the @target_version format.
+ *
+ * Warning: LUKS header loss is possible. See bd_crypto_luks_header_backup() and bd_crypto_luks_header_restore()
+ *
+ * Tech category: %BD_CRYPTO_TECH_LUKS-%BD_CRYPTO_TECH_MODE_MODIFY
+ */
+gboolean  bd_crypto_luks_convert (const gchar *device, BDCryptoLUKSVersion target_version, GError **error) {
+    return _bd_crypto_luks_convert (device, target_version, error);
+}
+
+
 static BDCryptoLUKSInfo* bd_crypto_luks_info_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_info' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_info' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_info' called, but not implemented!");
     return NULL;
@@ -1174,7 +1212,7 @@ BDCryptoLUKSInfo* bd_crypto_luks_info (const gchar *device, GError **error) {
 
 
 static BDCryptoBITLKInfo* bd_crypto_bitlk_info_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_bitlk_info' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_bitlk_info' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_bitlk_info' called, but not implemented!");
     return NULL;
@@ -1197,7 +1235,7 @@ BDCryptoBITLKInfo* bd_crypto_bitlk_info (const gchar *device, GError **error) {
 
 
 static BDCryptoIntegrityInfo* bd_crypto_integrity_info_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_integrity_info' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_integrity_info' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_integrity_info' called, but not implemented!");
     return NULL;
@@ -1220,7 +1258,7 @@ BDCryptoIntegrityInfo* bd_crypto_integrity_info (const gchar *device, GError **e
 
 
 static BDCryptoLUKSTokenInfo** bd_crypto_luks_token_info_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_luks_token_info' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_luks_token_info' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_luks_token_info' called, but not implemented!");
     return NULL;
@@ -1243,7 +1281,7 @@ BDCryptoLUKSTokenInfo** bd_crypto_luks_token_info (const gchar *device, GError *
 
 
 static gboolean  bd_crypto_integrity_format_stub (const gchar *device G_GNUC_UNUSED, const gchar *algorithm G_GNUC_UNUSED, gboolean wipe G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, BDCryptoIntegrityExtra *extra G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_integrity_format' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_integrity_format' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_integrity_format' called, but not implemented!");
     return FALSE;
@@ -1275,7 +1313,7 @@ gboolean  bd_crypto_integrity_format (const gchar *device, const gchar *algorith
 
 
 static gboolean  bd_crypto_integrity_open_stub (const gchar *device G_GNUC_UNUSED, const gchar *name G_GNUC_UNUSED, const gchar *algorithm G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, BDCryptoIntegrityOpenFlags flags G_GNUC_UNUSED, BDCryptoIntegrityExtra *extra G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_integrity_open' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_integrity_open' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_integrity_open' called, but not implemented!");
     return FALSE;
@@ -1305,7 +1343,7 @@ gboolean  bd_crypto_integrity_open (const gchar *device, const gchar *name, cons
 
 
 static gboolean  bd_crypto_integrity_close_stub (const gchar *integrity_device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_integrity_close' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_integrity_close' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_integrity_close' called, but not implemented!");
     return FALSE;
@@ -1328,7 +1366,7 @@ gboolean  bd_crypto_integrity_close (const gchar *integrity_device, GError **err
 
 
 static gboolean  bd_crypto_keyring_add_key_stub (const gchar *key_desc G_GNUC_UNUSED, const guint8 *key_data G_GNUC_UNUSED, gsize data_len G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_keyring_add_key' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_keyring_add_key' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_keyring_add_key' called, but not implemented!");
     return FALSE;
@@ -1353,7 +1391,7 @@ gboolean  bd_crypto_keyring_add_key (const gchar *key_desc, const guint8 *key_da
 
 
 static gboolean  bd_crypto_device_seems_encrypted_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_device_seems_encrypted' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_device_seems_encrypted' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_device_seems_encrypted' called, but not implemented!");
     return FALSE;
@@ -1390,7 +1428,7 @@ gboolean  bd_crypto_device_seems_encrypted (const gchar *device, GError **error)
 
 
 static gboolean  bd_crypto_tc_open_stub (const gchar *device G_GNUC_UNUSED, const gchar *name G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, const gchar **keyfiles G_GNUC_UNUSED, gboolean hidden G_GNUC_UNUSED, gboolean system G_GNUC_UNUSED, gboolean veracrypt G_GNUC_UNUSED, guint32 veracrypt_pim G_GNUC_UNUSED, gboolean read_only G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_tc_open' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_tc_open' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_tc_open' called, but not implemented!");
     return FALSE;
@@ -1423,7 +1461,7 @@ gboolean  bd_crypto_tc_open (const gchar *device, const gchar *name, BDCryptoKey
 
 
 static gboolean  bd_crypto_tc_close_stub (const gchar *tc_device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_tc_close' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_tc_close' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_tc_close' called, but not implemented!");
     return FALSE;
@@ -1446,7 +1484,7 @@ gboolean  bd_crypto_tc_close (const gchar *tc_device, GError **error) {
 
 
 static gboolean  bd_crypto_escrow_device_stub (const gchar *device G_GNUC_UNUSED, const gchar *passphrase G_GNUC_UNUSED, const gchar *cert_data G_GNUC_UNUSED, const gchar *directory G_GNUC_UNUSED, const gchar *backup_passphrase G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_escrow_device' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_escrow_device' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_escrow_device' called, but not implemented!");
     return FALSE;
@@ -1473,7 +1511,7 @@ gboolean  bd_crypto_escrow_device (const gchar *device, const gchar *passphrase,
 
 
 static gboolean  bd_crypto_bitlk_open_stub (const gchar *device G_GNUC_UNUSED, const gchar *name G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, gboolean read_only G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_bitlk_open' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_bitlk_open' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_bitlk_open' called, but not implemented!");
     return FALSE;
@@ -1501,7 +1539,7 @@ gboolean  bd_crypto_bitlk_open (const gchar *device, const gchar *name, BDCrypto
 
 
 static gboolean  bd_crypto_bitlk_close_stub (const gchar *bitlk_device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_bitlk_close' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_bitlk_close' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_bitlk_close' called, but not implemented!");
     return FALSE;
@@ -1524,7 +1562,7 @@ gboolean  bd_crypto_bitlk_close (const gchar *bitlk_device, GError **error) {
 
 
 static gboolean  bd_crypto_fvault2_open_stub (const gchar *device G_GNUC_UNUSED, const gchar *name G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, gboolean read_only G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_fvault2_open' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_fvault2_open' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_fvault2_open' called, but not implemented!");
     return FALSE;
@@ -1552,7 +1590,7 @@ gboolean  bd_crypto_fvault2_open (const gchar *device, const gchar *name, BDCryp
 
 
 static gboolean  bd_crypto_fvault2_close_stub (const gchar *fvault2_device G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_crypto_fvault2_close' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_fvault2_close' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_crypto_fvault2_close' called, but not implemented!");
     return FALSE;
@@ -1574,6 +1612,124 @@ gboolean  bd_crypto_fvault2_close (const gchar *fvault2_device, GError **error)
 }
 
 
+static gboolean  bd_crypto_opal_is_supported_stub (const gchar *device G_GNUC_UNUSED, GError **error) {
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_opal_is_supported' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_crypto_opal_is_supported' called, but not implemented!");
+    return FALSE;
+}
+
+static gboolean  (*_bd_crypto_opal_is_supported) (const gchar *device, GError **error) = bd_crypto_opal_is_supported_stub;
+
+/**
+ * bd_crypto_opal_is_supported:
+ * @device: device to check for OPAL support
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: %TRUE if the given @device supports OPAL or %FALSE if not or
+ * failed to determine (the @error is populated with the error in such
+ * cases).
+ *
+ * Tech category: %BD_CRYPTO_TECH_SED_OPAL-%BD_CRYPTO_TECH_MODE_QUERY
+ */
+gboolean  bd_crypto_opal_is_supported (const gchar *device, GError **error) {
+    return _bd_crypto_opal_is_supported (device, error);
+}
+
+
+static gboolean  bd_crypto_opal_wipe_device_stub (const gchar *device G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, GError **error) {
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_opal_wipe_device' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_crypto_opal_wipe_device' called, but not implemented!");
+    return FALSE;
+}
+
+static gboolean  (*_bd_crypto_opal_wipe_device) (const gchar *device, BDCryptoKeyslotContext *context, GError **error) = bd_crypto_opal_wipe_device_stub;
+
+/**
+ * bd_crypto_opal_wipe_device:
+ * @device: LUKS HW-OPAL device to wipe
+ * @context: OPAL admin passphrase context
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: whether @device was successfully wiped or not.
+ *
+ * Supported @context types for this function: passphrase
+ *
+ * Tech category: %BD_CRYPTO_TECH_SED_OPAL-%BD_CRYPTO_TECH_MODE_MODIFY
+ */
+gboolean  bd_crypto_opal_wipe_device (const gchar *device, BDCryptoKeyslotContext *context, GError **error) {
+    return _bd_crypto_opal_wipe_device (device, context, error);
+}
+
+
+static gboolean  bd_crypto_opal_format_stub (const gchar *device G_GNUC_UNUSED, const gchar *cipher G_GNUC_UNUSED, guint64 key_size G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, guint64 min_entropy G_GNUC_UNUSED, BDCryptoLUKSHWEncryptionType hw_encryption G_GNUC_UNUSED, BDCryptoKeyslotContext *opal_context G_GNUC_UNUSED, BDCryptoLUKSExtra *extra G_GNUC_UNUSED, GError **error) {
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_opal_format' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_crypto_opal_format' called, but not implemented!");
+    return FALSE;
+}
+
+static gboolean  (*_bd_crypto_opal_format) (const gchar *device, const gchar *cipher, guint64 key_size, BDCryptoKeyslotContext *context, guint64 min_entropy, BDCryptoLUKSHWEncryptionType hw_encryption, BDCryptoKeyslotContext *opal_context, BDCryptoLUKSExtra *extra, GError **error) = bd_crypto_opal_format_stub;
+
+/**
+ * bd_crypto_opal_format:
+ * @device: a device to format as LUKS HW-OPAL
+ * @cipher: (nullable): cipher specification (type-mode, e.g. "aes-xts-plain64") or %NULL to use the default
+ * @key_size: size of the volume key in bits or 0 to use the default
+ * @context: key slot context (passphrase/keyfile/token...) for this LUKS device
+ * @min_entropy: minimum random data entropy (in bits) required to format @device as LUKS
+ * @hw_encryption: type of hardware encryption (SW+HW or HW only)
+ * @opal_context: OPAL admin passphrase
+ * @extra: (nullable): extra arguments for LUKS format creation
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Formats the given @device as LUKS HW-OPAL according to the other parameters given. If
+ * @min_entropy is specified (greater than 0), the function waits for enough
+ * entropy to be available in the random data pool (WHICH MAY POTENTIALLY TAKE
+ * FOREVER).
+ *
+ * Supported @context types for this function: passphrase, key file
+ * Supported @opal_context types for this function: passphrase
+ *
+ * Returns: whether the given @device was successfully formatted as LUKS HW-OPAL or not
+ * (the @error contains the error in such cases)
+ *
+ * Tech category: %BD_CRYPTO_TECH_LUKS-%BD_CRYPTO_TECH_MODE_CREATE
+ */
+gboolean  bd_crypto_opal_format (const gchar *device, const gchar *cipher, guint64 key_size, BDCryptoKeyslotContext *context, guint64 min_entropy, BDCryptoLUKSHWEncryptionType hw_encryption, BDCryptoKeyslotContext *opal_context, BDCryptoLUKSExtra *extra, GError **error) {
+    return _bd_crypto_opal_format (device, cipher, key_size, context, min_entropy, hw_encryption, opal_context, extra, error);
+}
+
+
+static gboolean  bd_crypto_opal_reset_device_stub (const gchar *device G_GNUC_UNUSED, BDCryptoKeyslotContext *context G_GNUC_UNUSED, GError **error) {
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_crypto_opal_reset_device' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_crypto_opal_reset_device' called, but not implemented!");
+    return FALSE;
+}
+
+static gboolean  (*_bd_crypto_opal_reset_device) (const gchar *device, BDCryptoKeyslotContext *context, GError **error) = bd_crypto_opal_reset_device_stub;
+
+/**
+ * bd_crypto_opal_reset_device:
+ * @device: LUKS HW-OPAL device to run PSID reset on
+ * @context: PSID context
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: whether PSI reset on @device was successful or not.
+ *
+ * Warning: PSID reset will remove all data from @device!
+ *
+ * Supported @context types for this function: passphrase, key file
+ *
+ * Tech category: %BD_CRYPTO_TECH_SED_OPAL-%BD_CRYPTO_TECH_MODE_MODIFY
+ */
+gboolean  bd_crypto_opal_reset_device (const gchar *device, BDCryptoKeyslotContext *context, GError **error) {
+    return _bd_crypto_opal_reset_device (device, context, error);
+}
+
+
 static gpointer load_crypto_from_plugin(const gchar *so_name) {
     void *handle = NULL;
     char *error = NULL;
@@ -1706,6 +1862,11 @@ static gpointer load_crypto_from_plugin(const gchar *so_name) {
     if ((error = dlerror()) != NULL)
         bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_crypto_luks_set_uuid: %s", error);
 
+    dlerror();
+    * (void**) (&_bd_crypto_luks_convert) = dlsym(handle, "bd_crypto_luks_convert");
+    if ((error = dlerror()) != NULL)
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_crypto_luks_convert: %s", error);
+
     dlerror();
     * (void**) (&_bd_crypto_luks_info) = dlsym(handle, "bd_crypto_luks_info");
     if ((error = dlerror()) != NULL)
@@ -1786,6 +1947,26 @@ static gpointer load_crypto_from_plugin(const gchar *so_name) {
     if ((error = dlerror()) != NULL)
         bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_crypto_fvault2_close: %s", error);
 
+    dlerror();
+    * (void**) (&_bd_crypto_opal_is_supported) = dlsym(handle, "bd_crypto_opal_is_supported");
+    if ((error = dlerror()) != NULL)
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_crypto_opal_is_supported: %s", error);
+
+    dlerror();
+    * (void**) (&_bd_crypto_opal_wipe_device) = dlsym(handle, "bd_crypto_opal_wipe_device");
+    if ((error = dlerror()) != NULL)
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_crypto_opal_wipe_device: %s", error);
+
+    dlerror();
+    * (void**) (&_bd_crypto_opal_format) = dlsym(handle, "bd_crypto_opal_format");
+    if ((error = dlerror()) != NULL)
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_crypto_opal_format: %s", error);
+
+    dlerror();
+    * (void**) (&_bd_crypto_opal_reset_device) = dlsym(handle, "bd_crypto_opal_reset_device");
+    if ((error = dlerror()) != NULL)
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_crypto_opal_reset_device: %s", error);
+
     return handle;
 }
 
@@ -1815,6 +1996,7 @@ static gboolean unload_crypto (gpointer handle) {
     _bd_crypto_luks_header_restore = bd_crypto_luks_header_restore_stub;
     _bd_crypto_luks_set_label = bd_crypto_luks_set_label_stub;
     _bd_crypto_luks_set_uuid = bd_crypto_luks_set_uuid_stub;
+    _bd_crypto_luks_convert = bd_crypto_luks_convert_stub;
     _bd_crypto_luks_info = bd_crypto_luks_info_stub;
     _bd_crypto_bitlk_info = bd_crypto_bitlk_info_stub;
     _bd_crypto_integrity_info = bd_crypto_integrity_info_stub;
@@ -1831,6 +2013,10 @@ static gboolean unload_crypto (gpointer handle) {
     _bd_crypto_bitlk_close = bd_crypto_bitlk_close_stub;
     _bd_crypto_fvault2_open = bd_crypto_fvault2_open_stub;
     _bd_crypto_fvault2_close = bd_crypto_fvault2_close_stub;
+    _bd_crypto_opal_is_supported = bd_crypto_opal_is_supported_stub;
+    _bd_crypto_opal_wipe_device = bd_crypto_opal_wipe_device_stub;
+    _bd_crypto_opal_format = bd_crypto_opal_format_stub;
+    _bd_crypto_opal_reset_device = bd_crypto_opal_reset_device_stub;
 
     dlerror();
     * (void**) (&close_fn) = dlsym(handle, "bd_crypto_close");
