@@ -60,7 +60,7 @@ GType  bd_loop_info_get_type () {
 }
 
 static gboolean  bd_loop_is_tech_avail_stub (BDLoopTech tech G_GNUC_UNUSED, guint64 mode G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_loop_is_tech_avail' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_loop_is_tech_avail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_loop_is_tech_avail' called, but not implemented!");
     return FALSE;
@@ -83,7 +83,7 @@ gboolean  bd_loop_is_tech_avail (BDLoopTech tech, guint64 mode, GError **error)
 
 
 static BDLoopInfo* bd_loop_info_stub (const gchar *loop G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_loop_info' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_loop_info' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_loop_info' called, but not implemented!");
     return NULL;
@@ -106,7 +106,7 @@ BDLoopInfo* bd_loop_info (const gchar *loop, GError **error) {
 
 
 static gchar* bd_loop_get_loop_name_stub (const gchar *file G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_loop_get_loop_name' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_loop_get_loop_name' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_loop_get_loop_name' called, but not implemented!");
     return NULL;
@@ -129,7 +129,7 @@ gchar* bd_loop_get_loop_name (const gchar *file, GError **error) {
 
 
 static gboolean  bd_loop_setup_stub (const gchar *file G_GNUC_UNUSED, guint64 offset G_GNUC_UNUSED, guint64 size G_GNUC_UNUSED, gboolean read_only G_GNUC_UNUSED, gboolean part_scan G_GNUC_UNUSED, guint64 sector_size G_GNUC_UNUSED, const gchar **loop_name G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_loop_setup' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_loop_setup' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_loop_setup' called, but not implemented!");
     return FALSE;
@@ -158,7 +158,7 @@ gboolean  bd_loop_setup (const gchar *file, guint64 offset, guint64 size, gboole
 
 
 static gboolean  bd_loop_setup_from_fd_stub (gint fd G_GNUC_UNUSED, guint64 offset G_GNUC_UNUSED, guint64 size G_GNUC_UNUSED, gboolean read_only G_GNUC_UNUSED, gboolean part_scan G_GNUC_UNUSED, guint64 sector_size G_GNUC_UNUSED, const gchar **loop_name G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_loop_setup_from_fd' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_loop_setup_from_fd' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_loop_setup_from_fd' called, but not implemented!");
     return FALSE;
@@ -187,7 +187,7 @@ gboolean  bd_loop_setup_from_fd (gint fd, guint64 offset, guint64 size, gboolean
 
 
 static gboolean  bd_loop_teardown_stub (const gchar *loop G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_loop_teardown' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_loop_teardown' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_loop_teardown' called, but not implemented!");
     return FALSE;
@@ -210,7 +210,7 @@ gboolean  bd_loop_teardown (const gchar *loop, GError **error) {
 
 
 static gboolean  bd_loop_set_autoclear_stub (const gchar *loop G_GNUC_UNUSED, gboolean autoclear G_GNUC_UNUSED, GError **error) {
-    g_critical ("The function 'bd_loop_set_autoclear' called, but not implemented!");
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_loop_set_autoclear' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_loop_set_autoclear' called, but not implemented!");
     return FALSE;
