@@ -149,7 +149,7 @@ GType  bd_nvme_lba_format_get_type () {
  * @features: features and capabilities present for this namespace, see #BDNVMENamespaceFeature.
  * @format_progress_remaining: The percentage value remaining of a format operation in progress.
  * @write_protected: %TRUE if the namespace is currently write protected and all write access to the namespace shall fail.
- * @lba_formats: (array zero-terminated=1) (element-type BDNVMELBAFormat): A list of supported LBA Formats.
+ * @lba_formats: (array zero-terminated=1): A list of supported LBA Formats.
  * @current_lba_format: A LBA Format currently used for the namespace. Contains zeroes in case of
  *                      an invalid or no supported LBA Format reported.
  */
@@ -411,7 +411,7 @@ GType  bd_nvme_self_test_log_entry_get_type () {
  * BDNVMESelfTestLog:
  * @current_operation: Current running device self-test operation. There's no corresponding record in @entries for a device self-test operation that is in progress.
  * @current_operation_completion: Percentage of the currently running device self-test operation. Only valid when @current_operation is other than #BD_NVME_SELF_TEST_ACTION_NOT_RUNNING.
- * @entries: (array zero-terminated=1) (element-type BDNVMESelfTestLogEntry): Self-test log entries for the last 20 operations, sorted from newest (first element) to oldest.
+ * @entries: (array zero-terminated=1): Self-test log entries for the last 20 operations, sorted from newest (first element) to oldest.
  */
 /**
  * bd_nvme_self_test_log_free: (skip)
@@ -536,6 +536,29 @@ GType  bd_nvme_sanitize_log_get_type () {
     return type;
 }
 
+static gboolean  bd_nvme_is_tech_avail_stub (BDNVMETech tech G_GNUC_UNUSED, G_GNUC_UNUSED guint64 mode G_GNUC_UNUSED, GError **error) {
+    bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_nvme_is_tech_avail' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_nvme_is_tech_avail' called, but not implemented!");
+    return FALSE;
+}
+
+static gboolean  (*_bd_nvme_is_tech_avail) (BDNVMETech tech, G_GNUC_UNUSED guint64 mode, GError **error) = bd_nvme_is_tech_avail_stub;
+
+/**
+ * bd_nvme_is_tech_avail:
+ * @tech: the queried tech
+ * @mode: a bit mask of queried modes of operation (#BDNVMETechMode) for @tech
+ * @error: (out) (nullable): place to store error (details about why the @tech-@mode combination is not available)
+ *
+ * Returns: whether the @tech-@mode combination is available -- supported by the
+ *          plugin implementation and having all the runtime dependencies available
+ */
+gboolean  bd_nvme_is_tech_avail (BDNVMETech tech, G_GNUC_UNUSED guint64 mode, GError **error) {
+    return _bd_nvme_is_tech_avail (tech, mode, error);
+}
+
+
 static const gchar * bd_nvme_self_test_result_to_string_stub (BDNVMESelfTestResult result G_GNUC_UNUSED, GError **error) {
     bd_utils_log_format (BD_UTILS_LOG_CRIT, "The function 'bd_nvme_self_test_result_to_string' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
@@ -1173,6 +1196,11 @@ static gpointer load_nvme_from_plugin(const gchar *so_name) {
     }
     init_fn = NULL;
 
+    dlerror();
+    * (void**) (&_bd_nvme_is_tech_avail) = dlsym(handle, "bd_nvme_is_tech_avail");
+    if ((error = dlerror()) != NULL)
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_nvme_is_tech_avail: %s", error);
+
     dlerror();
     * (void**) (&_bd_nvme_self_test_result_to_string) = dlsym(handle, "bd_nvme_self_test_result_to_string");
     if ((error = dlerror()) != NULL)
@@ -1275,6 +1303,7 @@ static gboolean unload_nvme (gpointer handle) {
     char *error = NULL;
     gboolean (*close_fn) (void) = NULL;
 
+    _bd_nvme_is_tech_avail = bd_nvme_is_tech_avail_stub;
     _bd_nvme_self_test_result_to_string = bd_nvme_self_test_result_to_string_stub;
     _bd_nvme_get_controller_info = bd_nvme_get_controller_info_stub;
     _bd_nvme_get_namespace_info = bd_nvme_get_namespace_info_stub;
