@@ -43,6 +43,18 @@ typedef enum {
     BD_NVME_TECH_MODE_INITIATOR    = 1 << 2,
 } BDNVMETechMode;
 
+/**
+ * bd_nvme_is_tech_avail:
+ * @tech: the queried tech
+ * @mode: a bit mask of queried modes of operation (#BDNVMETechMode) for @tech
+ * @error: (out) (nullable): place to store error (details about why the @tech-@mode combination is not available)
+ *
+ * Returns: whether the @tech-@mode combination is available -- supported by the
+ *          plugin implementation and having all the runtime dependencies available
+ */
+gboolean  bd_nvme_is_tech_avail (BDNVMETech tech, G_GNUC_UNUSED guint64 mode, GError **error);
+
+
 
 typedef enum {
     BD_NVME_CTRL_FEAT_MULTIPORT           = 1 << 0,
@@ -231,7 +243,7 @@ typedef struct BDNVMENamespaceInfo {
  * @features: features and capabilities present for this namespace, see #BDNVMENamespaceFeature.
  * @format_progress_remaining: The percentage value remaining of a format operation in progress.
  * @write_protected: %TRUE if the namespace is currently write protected and all write access to the namespace shall fail.
- * @lba_formats: (array zero-terminated=1) (element-type BDNVMELBAFormat): A list of supported LBA Formats.
+ * @lba_formats: (array zero-terminated=1): A list of supported LBA Formats.
  * @current_lba_format: A LBA Format currently used for the namespace. Contains zeroes in case of
  *                      an invalid or no supported LBA Format reported.
  */
@@ -493,7 +505,7 @@ typedef struct BDNVMESelfTestLog {
  * BDNVMESelfTestLog:
  * @current_operation: Current running device self-test operation. There's no corresponding record in @entries for a device self-test operation that is in progress.
  * @current_operation_completion: Percentage of the currently running device self-test operation. Only valid when @current_operation is other than #BD_NVME_SELF_TEST_ACTION_NOT_RUNNING.
- * @entries: (array zero-terminated=1) (element-type BDNVMESelfTestLogEntry): Self-test log entries for the last 20 operations, sorted from newest (first element) to oldest.
+ * @entries: (array zero-terminated=1): Self-test log entries for the last 20 operations, sorted from newest (first element) to oldest.
  */
 /**
  * bd_nvme_self_test_log_free: (skip)
